/**
 * Proiectarea Algoritmilor, 2014
 * Lab 8: Drumuri minime
 * 
 * @author  Radu Iacob
 * @email   radu.iacob23@gmail.com
 */
 
#ifndef GRAPH_H
#define GRAPH_H

#include "Utils.h"
#include "Node.h"

class Graph {

    std::vector<Node*> Nodes;
    std::vector<std::vector<std::pair<Node*, int> > > Edges;

public:

    Graph();
    Graph(const std::string& input_graph);

    /** Return a vector with all the nodes in the graph. **/
    std::vector<Node*>&                  get_nodes();
    
    /** Return a vector with all the edges associated with a given node. **/
    std::vector<std::pair<Node*, int> >& get_edges(Node& node);

    /* Removes all temporary information related to the state of the graph
     *	(for example it sets every node as being unvisited)
     */
    void reset();
    
    /** Returns the coresponding node, given the name of the city. **/
    Node* get_node(const std::string& city);

    uint get_node_count() const;

    /** Functions for building the graph **/
    void insert_node(Node* node);
    void insert_edge(Node* node1, Node* node2, int cost);

    void clear_graph();
    virtual ~Graph();

	/** Convenience overload of the IO operators for pretty printing/reading a graph **/
    friend std::istream& operator>> (std::istream& in , Graph& node);
    friend std::ostream& operator<< (std::ostream& out, Graph& node);

private:

    void init_from_file(const std::string& input_graph);

    /** Maps each city name to a specific node in the graph **/
    std::map<std::string, Node*> nodeDict;

    void register_node(Node& node);

    /** Check if a node has an invalid id. **/
    void check_node(Node& node) const;
};

#endif
