/**
 * Proiectarea Algoritmilor, 2014
 * Lab 8: Drumuri minime
 *
 * @author  Radu Iacob
 * @email   radu.iacob23@gmail.com
 */

#include "MinCostAlgo.h"

static const std::string dataSet1("./test1.in"); /* positive edges */
static const std::string dataSet2("./test2.in"); /* negative edges */
static const std::string dataSet3("./test3.in"); /* negative cycle */

static const std::string start_city("Bucuresti");
static const std::string target_city("Paris");

void check_args(int argc, char* argv[]);
void print_distance_vector(Graph& graph, std::vector<int>& distance);

enum Task{
    ROY_FLOYD,      // Task 1
    DIJKSTRA,       // Task 2
    BELLMAN_FORD    // Task 3 + 4
};

/* TODO: change dataSet and currentTask */
static Task currentTask = ROY_FLOYD;
static const std::string currentDataSet = dataSet1;

int main(int argc, char* argv[])
{
    /* If you want, you can specify the task at runtime using
     * ./lab8 task_id 
     */
    check_args(argc, argv);

    Graph graph(currentDataSet);
    Node* source   = graph.get_node(start_city);
    Node* dest     = graph.get_node(target_city);
    int node_count = graph.get_node_count();

    switch (currentTask) {

        case ROY_FLOYD:
        {
            std::cout << "Task1 RoyFloyd: \n";
            Matrix<int>   costMatrix(node_count, node_count);
            Matrix<Node*> detourMatrix(node_count, node_count);

            RoyFloyd(graph, costMatrix, detourMatrix);
            MinPath(graph, costMatrix, detourMatrix, source, dest);

            break;
        }

        case DIJKSTRA:
        {
            std::cout << "Task2 Dijkstra: \n";
            std::vector<int> distance = Dijkstra(graph, source);
            print_distance_vector(graph, distance);

            break;
        }

        case BELLMAN_FORD:
        {
            std::cout << "Task3 BellmanFord: \n";
            std::vector<int> distance = BellmanFord(graph, source);
            print_distance_vector(graph, distance);

            break;
        }

        default: 
        {
            std::cout << "Undefined task.\n";
            break;
        }
    }

    graph.clear_graph();
}

void check_args(int argc, char* argv[]) {
    if (argc != 1) {        
        int task_id;
        sscanf(argv[1], "%d", &task_id);
        currentTask = static_cast<Task>(task_id);
    }
}

void print_distance_vector(Graph& graph, std::vector<int>& distance)
{
    std::vector< Node* >& nodes = graph.get_nodes();
    std::cout << "Result:\n";
    for(uint i = 0, sz = nodes.size(); i < sz; ++i){
        std::cout << "< " << nodes[i]->get_city() << " , " << distance[i] << " >\n";
    }
}
