/**
 * Proiectarea Algoritmilor, 2014
 * Lab 8: Drumuri minime
 *
 * @author  Radu Iacob
 * @email   radu.iacob23@gmail.com
 */

package graph;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Scanner;

import graph.Node;

public class Graph {

    ArrayList<Node> nodes            = new ArrayList<Node>();
    ArrayList<ArrayList<Edge>> edges = new ArrayList<ArrayList<Edge>>();

    public Graph() {
    }

    public Graph(String inputFile) {
        initFromFile(inputFile);
    }

    /**
     * @return a list with every node in the graph
     */
    public ArrayList<Node> getNodes() {
        return nodes;
    }

    /**
     * @param node
     * @return a list of every out-going edge of a given node
     */
    public ArrayList<Edge> getEdges(Node node) {
        return edges.get(node.getId());
    }

    public int getNodeCount() {
        return nodes.size();
    }

    /**
     * This method removes all the temporary information related to the state
     * of the graph. (for example it sets every node as being unvisited)
     */
    public void reset() {
        for (Node node : nodes) {
            node.reset();
        }
    }

    public void initFromFile(String inputFile) {

        clearGraph();
        Scanner scanner;
        try {
            scanner = new Scanner(new File(inputFile));
            readData(scanner);
            scanner.close();
        } catch (FileNotFoundException e) {
            throw new RuntimeException("Check file path.(invalid scanner object)");
        }
    }

    /**
     * Function to parse a custom graph file for the lab </br>
     *
     * Input Format: </br>
     * N = number of nodes </br> 
     * N lines with each city name </br> 
     * 
     * M = number of edges </br>
     * M lines with Node_i Node_j cost_Between_Nodei_Nodej -- list of edges </br>
     * 
     * @param filename
     */
    private void readData(Scanner scanner) {

        int numNodes = scanner.nextInt();
        for (int i = 0; i < numNodes; ++i) {
            String city   = scanner.next();
            Node new_node = new Node(city, i);
            insertNode(new_node);
        }

        int numEdges = scanner.nextInt();
        int node1, node2, cost;

        for (int i = 0; i < numEdges; ++i) {
            node1 = scanner.nextInt();
            node2 = scanner.nextInt();
            cost  = scanner.nextInt();
            insertEdge(nodes.get(node1), nodes.get(node2), cost);
        }
    }

    /**
     * Adds the argument node to the graph.
     * @param node
     */
    public void insertNode(Node node) {
        nodes.add(node);
        edges.add(new ArrayList<Edge>());
        registerNode(node);
    }

    /**
     * Creates a link between the two specified argument nodes.
     * @param node1
     * @param node2
     */
    public void insertEdge(Node node1, Node node2, Integer cost) {
        edges.get(node1.getId()).add(new Edge(node2, cost));
    }

    public void clearGraph() {
        nodes.clear();
        edges.clear();
        nodeDict.clear();
    }

    /** Maps each city name to a specific node in the graph **/
    private HashMap<String, Node> nodeDict = new HashMap<String, Node>();

    /** 
     * @param  city
     * @return the specific node mapped to the given parameter city 
     **/
    public Node getNode(String city) {
        return nodeDict.get(city);
    }

    private void registerNode(Node node) {
        nodeDict.put(node.getCity(), node);
    }
    
    public static final int MAX_VALUE = Integer.MAX_VALUE;
    
    public String toString() {    
        StringBuilder ans = new StringBuilder("Print Graph:\n");
        
        for (Node node : nodes) {
            ans.append("OutEdges for " + node.toString() + " -> ");
            for (Edge edge : getEdges(node)) {
                ans.append(edge + " |");
            }
            ans.append("\n");
        }
        ans.append("\n");

        return ans.toString();
    }
}
